// ParamBase is an abstract base class for parameter management.
// It is used by the PluginManager to store and manipulate a vector of polymorphic parameters.
// Subclasses must define the GetMixedValue() and SetMixedValue() functions else they cannot be instantiated.
//
// 2019-08-22 ffgl@subpixels.com
#pragma once
#include <string>
#include "ffgl/FFGL.h"

namespace ffgl
{

class ParamBase
{
protected:
	std::string name;
	ParameterType type;
	std::string uniformName;
	GLint uniformLocation;

	ParamBase( const std::string& name, ParameterType type );
	ParamBase( const std::string& name, ParameterType type, const std::string& uniformName );

public:
	bool doAutoConversion = true;

	~ParamBase();

	const std::string& GetName() const;
	ParameterType GetType() const;

	virtual void Reset() = 0;
	virtual const std::string& GetDisplay() const = 0;
	virtual FFMixed GetMixedDefault() const = 0;
	virtual FFMixed GetMixedValue() const = 0;
	virtual FFResult SetMixedValue( const FFMixed mixed ) = 0;

	const std::string& GetUniformName() const;
	void SetUniformName(const std::string& name);

	const GLint GetUniformLocation() const;
	void SetUniformLocation(GLint location);
};

}//End namespace ffgl
